@@ -3,9 +3,9 @@
 ## Author: Brice Ozenne
 ## Created: jun 27 2019 (10:43) 
 ## Version: 
-## Last-Updated: Sep  6 2023 (09:53) 
+## Last-Updated: May 14 2025 (15:29) 
 ##           By: Thomas Alexander Gerds
-##     Update #: 935
+##     Update #: 1132
 ##----------------------------------------------------------------------
 ## 
 ### Commentary: 
@@ -26,10 +26,8 @@ ATE_TI <- function(object.event,
                    contrasts,
                    allContrasts,
                    times,
-                   landmark,
                    cause,
                    level.censoring,
-                   n.contrasts,
                    levels,
                    n.censor,
                    estimator,
@@ -37,11 +35,12 @@ ATE_TI <- function(object.event,
                    eventVar.status,
                    censorVar.time,
                    censorVar.status,
-                   type.multistate,
                    return.iid.nuisance,
                    data.index,
                    method.iid,
                    product.limit,
+                   store,
+                   verbose,
                    ...){
 
     tol <- 1e-12 ## difference in jump time must be above tol
@@ -107,14 +106,14 @@ ATE_TI <- function(object.event,
                                    )
 
         ## *** jump time for the censoring process
-        time.jumpC <- sort(mydata[[censorVar.time]][(mydata[[censorVar.status]] == level.censoring)])
+        time.jumpC <- unique(sort(mydata[[censorVar.time]][(mydata[[censorVar.status]] == level.censoring)]))
         index.obsSINDEXjumpC <- do.call(cbind,lapply(times, function(tau){
             prodlim::sindex(jump.times = time.jumpC, eval.times = pmin(mydata[[censorVar.time]],tau)-tol)
         }))
         index.lastjumpC <- max(index.obsSINDEXjumpC)
         time.jumpC <- time.jumpC[1:index.lastjumpC]
-
     }
+
     if(attr(estimator,"integral")){
         ## *** jump time index of the event time stopped at tau
         index.obsSINDEXjumpC.int <- do.call(cbind,lapply(times, function(tau){
@@ -126,7 +125,7 @@ ATE_TI <- function(object.event,
         beforeTau.nJumpC <- sapply(times, function(iTau){sum(time.jumpC <= iTau)})
         beforeTau.nJumpC.n0 <- beforeTau.nJumpC[beforeTau.nJumpC!=0]
     }
-    
+
     ## ** compute predictions
     ## *** treatment model
     if(attr(estimator,"IPTW")){
@@ -142,9 +141,9 @@ ATE_TI <- function(object.event,
 
     ## *** censoring model
     if(attr(estimator,"IPCW")){
-        iPred <- lapply(1:n.times, function(iT){
+        iPred <- lapply(1:n.times, function(iT){ ## iT <- 1
             1-predictRisk(object.censor, newdata = mydata, times = c(0,time.jumpC)[index.obsSINDEXjumpC[,iT]+1],
-                        diag = TRUE, product.limit = product.limit, iid = (method.iid==2)*return.iid.nuisance)
+                          diag = TRUE, product.limit = product.limit, iid = (method.iid==2)*return.iid.nuisance, store = store[c("data","iid")])
         })
             
         G.T_tau <- do.call(cbind,iPred)
@@ -201,7 +200,7 @@ ATE_TI <- function(object.event,
             }
             outRisk <- predictRisk(object.event, newdata = data.i, times = times,
                                    average.iid = factor, cause = cause,
-                                   product.limit = product.limit)
+                                   product.limit = product.limit, store = store[c("data","iid")])
 
             F1.ctf.tau[[iC]][ls.index.strata[[iC]],] <- outRisk
             if(return.iid.nuisance){
@@ -215,57 +214,121 @@ ATE_TI <- function(object.event,
         }
     }
 
-    ## ** Compute augmentation term
+    ## ** Compute augmentation term 
     if(attr(estimator,"integral")){
-        ## absolute risk at event times
-        predTempo <- predictRisk(object.event, newdata = mydata, times = c(times, time.jumpC), cause = cause, product.limit = product.limit,
-                                 iid = (method.iid==2)*return.iid.nuisance)
-        F1.tau <- predTempo[,1:n.times,drop=FALSE]
-        F1.jump <- predTempo[,n.times + (1:index.lastjumpC),drop=FALSE]
-        if((method.iid==2)*return.iid.nuisance){
-            out$store$iid.nuisance.outcome <- attr(predTempo,"iid")
-        }
+        ## integral is sum over individuals from 0 to min(T_i,\tau) of dM_i^C. It is therefore non-0 only if:
+        ## - some of the requested times are after the first censoring time 
+        ## - some of the censoring jump times are before the event times
+        ## this would correspond to index.lastjumpC>0 && any(beforeEvent.jumpC)
+        ## the first should be automatically enforced in ate_initArgs when creating attr(estimator,"integral")
         
-        ## survival at all jump of the censoring process
-        S.jump <- predictRisk(object.event, type = "survival", newdata = mydata, times = time.jumpC-tol, product.limit = product.limit,
-                              iid = (method.iid==2)*return.iid.nuisance)
-        if((method.iid==2)*return.iid.nuisance){
-            out$store$iid.nuisance.survival <- attr(S.jump,"iid")
-            attr(S.jump,"iid") <- NULL
-        }
-
-        ## martingale for the censoring process
-        ## at all times of jump of the censoring process
-        G.jump <- 1-predictRisk(object.censor, newdata = mydata, times = if(index.lastjumpC>1){c(0,time.jumpC[1:(index.lastjumpC-1)])}else{0},
-                                product.limit = product.limit, iid = (method.iid==2)*return.iid.nuisance)
-        
-        if(return.iid.nuisance && (method.iid==2)){
-            out$store$iid.nuisance.censoring <- -attr(G.jump,"iid")
-            attr(G.jump,"iid") <- NULL
-        }
-        
-        dN.jump <- do.call(rbind,lapply(1:n.obs, function(iObs){(mydata[[eventVar.time]][iObs] == time.jumpC)*(mydata[[eventVar.status]][iObs] == level.censoring)}))
-        dLambda.jump <- predictCox(object.censor, newdata = mydata, times = time.jumpC, type = "hazard", iid = (method.iid==2)*return.iid.nuisance)
-        if((method.iid==2)*return.iid.nuisance){
-            out$store$iid.nuisance.martingale <- dLambda.jump$hazard.iid
-        }
-
-        dM.jump <- dN.jump - dLambda.jump$hazard
-
-        ## integral
-        integrand <- matrix(0, nrow = n.obs, ncol = index.lastjumpC)
-        integrand2 <- matrix(0, nrow = n.obs, ncol = index.lastjumpC)
-        index.beforeEvent.jumpC <- which(beforeEvent.jumpC)
-        if(length(index.beforeEvent.jumpC)>0){
-            integrand[index.beforeEvent.jumpC] <- dM.jump[index.beforeEvent.jumpC] / (G.jump[index.beforeEvent.jumpC] * S.jump[index.beforeEvent.jumpC])
-            integrand2[index.beforeEvent.jumpC] <- F1.jump[index.beforeEvent.jumpC] * integrand[index.beforeEvent.jumpC]
-        }
-        integral <- rowCumSum(integrand)
-        integral2 <- rowCumSum(integrand2)
         augTerm <- matrix(0, nrow = n.obs, ncol = n.times)
-        augTerm[,beforeTau.nJumpC!=0] <- F1.tau[,beforeTau.nJumpC!=0,drop=FALSE] * integral[,beforeTau.nJumpC.n0,drop=FALSE] - integral2[,beforeTau.nJumpC.n0,drop=FALSE]
+
+        ## *** exclude individuals with event before the first censoring as they do not contribute to the integral term
+        if(method.iid==2){ ## slow but simple i.e. no subset
+            indexAll.obsIntegral <- 1:n.obs
+        }else{ 
+            indexAll.obsIntegral <- which(index.obsSINDEXjumpC.int[,n.times]>0) ## WARNING do not use index.obsSINDEXjumpC as it is evaluated just before the jump (i.e. time-tol)
+        }
+
+        ## *** split dataset to avoid large matrices
+        if(method.iid==2 || return.iid.nuisance || is.null(store$size.split)){
+            n.split <- 1
+        }else{
+            size.split <- store$size.split
+            nAll.obsIntegal <- NROW(indexAll.obsIntegral)
+            label.split <- cut(1:nAll.obsIntegal, breaks = ceiling(nAll.obsIntegal/size.split))
+            n.split <- length(levels(label.split))
+            if(verbose>1){
+                cat(" ")
+                pb <- txtProgressBar(max = n.split, style = 1)
+            }
+        }
+
+        for(iSplit in 1:n.split){ ## iSplit <- 1
+            if(n.split == 1){
+                index.obsIntegral <- indexAll.obsIntegral
+            }else{
+                if(verbose>1){
+                    setTxtProgressBar(pb = pb, value = iSplit)
+                }
+                index.obsIntegral <- indexAll.obsIntegral[label.split==levels(label.split)[iSplit]]
+            }
+            mydataIntegral <- mydata[index.obsIntegral,,drop=FALSE]
+            n.obsIntegral <- NROW(mydataIntegral)
+
+            ## *** evaluate survival functions (event, censoring)
+            ## absolute risk at event times and jump times of the censoring process
+            predTempo <- predictRisk(object.event, newdata = mydataIntegral, times = c(times, time.jumpC), cause = cause, product.limit = product.limit,
+                                     iid = (method.iid==2)*return.iid.nuisance, store = store[c("data","iid")])
+            F1.tau <- predTempo[,1:n.times,drop=FALSE]
+            F1.jump <- predTempo[,n.times + (1:index.lastjumpC),drop=FALSE]
+            if((method.iid==2)*return.iid.nuisance){
+                out$store$iid.nuisance.outcome <- attr(predTempo,"iid")
+            }
+        
+            ## survival at all jump of the censoring process
+            S.jump <- predictRisk(object.event, type = "survival", newdata = mydataIntegral, times = time.jumpC-tol, product.limit = product.limit,
+                                  iid = (method.iid==2)*return.iid.nuisance, store = store[c("data","iid")])
+            if((method.iid==2)*return.iid.nuisance){
+                out$store$iid.nuisance.survival <- attr(S.jump,"iid")
+                attr(S.jump,"iid") <- NULL
+            }
+
+            ## martingale for the censoring process
+            ## at all times of jump of the censoring process
+            G.jump <- 1-predictRisk(object.censor, newdata = mydataIntegral, times = if(index.lastjumpC>1){c(0,time.jumpC[1:(index.lastjumpC-1)])}else{0},
+                                    product.limit = product.limit, iid = (method.iid==2)*return.iid.nuisance, store = store[c("data","iid")])
+        
+            if(return.iid.nuisance && (method.iid==2)){
+                out$store$iid.nuisance.censoring <- -attr(G.jump,"iid")
+                attr(G.jump,"iid") <- NULL
+            }
+            dLambda.jump <- predictCox(object.censor, newdata = mydataIntegral, times = time.jumpC, type = "hazard", iid = (method.iid==2)*return.iid.nuisance, store = store[c("data","iid")])
+            if((method.iid==2)*return.iid.nuisance){
+                out$store$iid.nuisance.martingale <- dLambda.jump$hazard.iid
+            }
+
+            ## *** evaluate martingal w.r.t. the censoring mechanism
+            ## ## version 1: slow
+            ## dN.jump <- do.call(cbind,lapply(time.jumpC, function(iJump){(mydataIntegral[[eventVar.time]] == iJump)*(mydataIntegral[[eventVar.status]] == level.censoring)}))
+            ## dM.jump <- dN.jump - dLambda.jump$hazard
+
+            ## version 2: fast
+            dM.jump <- - dLambda.jump$hazard
+            indexTime.jumpCindiv <- match(mydataIntegral[[eventVar.time]],  time.jumpC) ## index of the individual event times matching the jumps
+            index.jumpCindiv <- intersect(which(!is.na(indexTime.jumpCindiv)), which(mydataIntegral[[eventVar.status]] == level.censoring)) ## index of the individual jumping at the right time and having a censoring event
+            ## range(which(dN.jump!=0) - sort(index.jumpCindiv + (indexTime.jumpCindiv[index.jumpCindiv]-1)*n.obsIntegral))
+            dM.jump[sort(index.jumpCindiv + (indexTime.jumpCindiv[index.jumpCindiv]-1)*n.obsIntegral)] <- 1 + dM.jump[sort(index.jumpCindiv + (indexTime.jumpCindiv[index.jumpCindiv]-1)*n.obsIntegral)]
+
+            ## *** evaluate integral
+            ## integral = \int_0^min(T_i,\tau) (F1(\tau|A_i,W_i)-F1(t|A_i,W_i)) / S(t|A_i,W_i) * dM_i^C(t)/Gc(t|A_i,W_i)
+            ##          = F1(\tau|A_i,W_i) \int_0^min(T_i,\tauf) dM_i^C(t) / (S(t|A_i,W_i) * Gc(t|A_i,W_i)) - \int_0^min(T_i,\tauf) F1(t|A_i,W_i) dM_i^C(t) / (S(t|A_i,W_i) * Gc(t|A_i,W_i))
+
+            ## ## version 1 (matrix product and sums): does not scale well with sample size
+            ## integrand <- matrix(0, nrow = n.obsIntegral, ncol = index.lastjumpC)
+            ## integrand2 <- matrix(0, nrow = n.obsIntegral, ncol = index.lastjumpC)
+            ## index.beforeEvent.jumpC <- which(beforeEvent.jumpC[index.obsIntegral,,drop=FALSE]) ## identify which increment to put in the integral, i.e. when the individual was still at risk of being censored
+            ## integrand[index.beforeEvent.jumpC] <- dM.jump[index.beforeEvent.jumpC] / (G.jump[index.beforeEvent.jumpC] * S.jump[index.beforeEvent.jumpC])
+            ## integrand2[index.beforeEvent.jumpC] <- F1.jump[index.beforeEvent.jumpC] * integrand[index.beforeEvent.jumpC]
+            ## integral <- rowCumSum(integrand)
+            ## integral2 <- rowCumSum(integrand2)
+            ## augTerm[index.obsIntegral,beforeTau.nJumpC!=0] <- F1.tau[,beforeTau.nJumpC!=0,drop=FALSE] * integral[,beforeTau.nJumpC.n0,drop=FALSE] - integral2[,beforeTau.nJumpC.n0,drop=FALSE]
+
+            ## version 2 (loop): scale ok with sample size
+            for(iObs in 1:n.obsIntegral){ ## iObs <- which(index.obsIntegral[iObs]==428)
+                iMax.jumpC <- index.obsSINDEXjumpC.int[index.obsIntegral[iObs],n.times]
+                if(iMax.jumpC==0){next} ## case where method.iid = 2
+                iIndex.tau <- pmin(beforeTau.nJumpC.n0, iMax.jumpC)
+                iIntegrand <- dM.jump[iObs,1:iMax.jumpC] / (G.jump[iObs,1:iMax.jumpC] * S.jump[iObs, 1:iMax.jumpC])
+                augTerm[index.obsIntegral[iObs],beforeTau.nJumpC!=0] <- F1.tau[iObs,beforeTau.nJumpC!=0,drop=FALSE] * cumsum(iIntegrand)[iIndex.tau] - cumsum(F1.jump[iObs,1:iMax.jumpC] * iIntegrand)[iIndex.tau]
+            }
+        }
+
+        if(n.split>1 && verbose>1){close(pb)}
+
     }
-       
+
     ## ** Compute individual contribution to the ATE + influence function for the G-formula
     for(iC in 1:n.contrasts){ ## iC <- 1
         if(attr(estimator,"export.GFORMULA")){
@@ -297,12 +360,10 @@ ATE_TI <- function(object.event,
             out$store$iid.IPTW[[iC]][data.index,] <- out$store$iid.IPTW[[iC]][data.index,]  + rowCenter_cpp(iIID.ate, center = iATE)/n.obs
         }
         if(attr(estimator,"export.AIPTW")){
-            if(attr(estimator,"IPCW")){
-                if(inherits(object.event,"wglm")){
-                    iIID.ate <- F1.ctf.tau[[iC]] + colMultiply_cpp(iW.IPCW * Y.tau - F1.ctf.tau[[iC]], scale = iW.IPTW[,iC])
-                }else{
-                    iIID.ate <- F1.ctf.tau[[iC]] + colMultiply_cpp(iW.IPCW * Y.tau - F1.ctf.tau[[iC]] + augTerm, scale = iW.IPTW[,iC])
-                }
+            if(attr(estimator,"integral")){ ## full augmentation
+                iIID.ate <- F1.ctf.tau[[iC]] + colMultiply_cpp(iW.IPCW * Y.tau - F1.ctf.tau[[iC]] + augTerm, scale = iW.IPTW[,iC])
+            }else if(inherits(object.event,"wglm") && attr(estimator,"IPCW")){ ## not full augmentation
+                iIID.ate <- F1.ctf.tau[[iC]] + colMultiply_cpp(iW.IPCW * Y.tau - F1.ctf.tau[[iC]], scale = iW.IPTW[,iC])
             }else{
                 iIID.ate <- F1.ctf.tau[[iC]] + colMultiply_cpp(Y.tau - F1.ctf.tau[[iC]], scale = iW.IPTW[,iC])
             }
@@ -315,8 +376,7 @@ ATE_TI <- function(object.event,
             out$store$iid.AIPTW[[iC]][data.index,] <- out$store$iid.AIPTW[[iC]][data.index,] + rowCenter_cpp(iIID.ate, center = iATE)/n.obs
         }
     }
-    
-    
+        
     ## ** save quantities useful for the calculation of iid.nuisance
     if(return.iid.nuisance){
         out$store$n.obs <- n.obs
@@ -352,83 +412,28 @@ ATE_TI <- function(object.event,
             out$store$beforeEvent.jumpC <- beforeEvent.jumpC
             out$store$beforeTau.nJumpC <- beforeTau.nJumpC
             out$store$index.obsSINDEXjumpC.int <- index.obsSINDEXjumpC.int
+            out$store$index.obsIntegral <- index.obsIntegral            
         }
     }
 
     ## ** Compute risk comparisons
-    out[c("diffRisk","ratioRisk")] <- ATE_COMPARISONS(out$meanRisk, TD = FALSE, allContrasts = allContrasts)
+    out[c("diffRisk","ratioRisk")] <- ATE_COMPARISONS(out$meanRisk, allContrasts = allContrasts)
     
     ## ** Export
     return(out)            
 }
 
-## * ATE_TD: compute average risk for time dependent covariates (using G-formula)
-ATE_TD <- function(object.event,
-                   mydata,
-                   formula,
-                   treatment,
-                   contrasts,
-                   allContrasts,
-                   times,
-                   landmark,
-                   cause,
-                   n.contrasts,
-                   levels,
-                   ...){
-
-    n.contrasts <- length(contrasts)
-
-    response <- eval(formula[[2]],envir=mydata)
-    time <- response[,"time"]
-    entry <- response[,"entry"]
-    if(inherits(x=object.event,what="coxph")){
-        riskhandler <- "predictRisk.coxphTD"
-    }else{
-        riskhandler <- "predictRisk.CSCTD"
-    }
-    ## prediction for the hypothetical worlds in which every subject is treated with the same treatment
-    out <- list(meanRisk = NULL,
-                diffRisk = NULL,
-                ratioRisk = NULL,
-                store = NULL)
-    out$meanRisk <- data.table::rbindlist(lapply(1:n.contrasts,function(i){
-        data.i <- mydata
-        data.i[[treatment]] <- factor(contrasts[i], levels = levels)
-        data.table::rbindlist(lapply(landmark,function(lm){
-            atrisk <- (entry <= lm & time >= lm)
-            risk.i <- colMeans(do.call(riskhandler,
-                                       args = list(object.event,
-                                                   newdata = data.i[atrisk,],
-                                                   times = times,
-                                                   cause = cause,
-                                                   landmark=lm,
-                                                   ...)))
-            data.table::data.table(estimator = "GFORMULA",
-                                   treatment=contrasts[[i]],
-                                   time=times,
-                                   landmark=lm,
-                                   estimate=risk.i)
-        }))
-    }))
-
-    ## ** Compute risk comparisons
-    out[c("diffRisk","ratioRisk")] <- ATE_COMPARISONS(out$meanRisk, TD = TRUE, allContrasts = allContrasts)
-
-    ## ** Export
-    return(out)
-}
-
 ## * ATE_COMPARISONS: compute average risk for time dependent covariates (using G-formula)
-ATE_COMPARISONS <- function(data, TD, allContrasts){
+ATE_COMPARISONS <- function(data, allContrasts){
     ## duplicate
-    dataA <- data.table::copy(data)
+    dataA <- copy(data)
     setnames(dataA, old = c("treatment","estimate"), new = c("A","estimate.A"))
-    dataB <- data.table::copy(data)
+    dataB <- copy(data)
     setnames(dataB, old = c("treatment","estimate"), new = c("B","estimate.B"))
 
     ## perform all pairwise combinations
     mdata <- do.call(rbind,apply(allContrasts, 2, function(iC){ merge(dataA[iC[1],.SD, on = "A"],dataB[iC[2],.SD, on = "B"],
-                                                                      by = c("estimator","time",if(TD){"landmark"}))
+                                                                      by = c("estimator","time"))
     })) ## iC <- c("T0","T1")
 
     ## re-order by estimator
