@@ -3,9 +3,9 @@
 ## author: Brice Ozenne
 ## created: feb 17 2017 (10:06) 
 ## Version: 
-## last-updated: Mar  7 2022 (08:31) 
-##           By: Thomas Alexander Gerds
-##     Update #: 1275
+## last-updated: Oct 16 2024 (09:24) 
+##           By: Brice Ozenne
+##     Update #: 1299
 ##----------------------------------------------------------------------
 ## 
 ### Commentary: 
@@ -358,10 +358,22 @@ autoplot.predictCox <- function(object,
             }
             status <- NULL
             if(first.derivative && ci){
-                if(is.null(object$vcov[[type]])){
+                if(is.null(object[[paste0(type,".iid")]])){
                     stop("Set argument \'iid\' to TRUE when calling predictCox to be able to display confidence intervals for the first derivative of the ",type,".\n")
                 }
-                attr(first.derivative,"vcov") <- object$vcov[[type]]
+                iIID <- object[[paste0(type,".iid")]]
+                ## compute variance-covariance matrix
+                attr(first.derivative,"vcov") <- lapply(1:(dim(iIID)[3]), function(iObs){
+                    if(dim(iIID)[2]==1){
+                        if(type=="lp"){
+                            return(sum(iIID[,iObs,]^2))
+                        }else{
+                            return(sum(iIID[,,iObs]^2))
+                        }
+                    }else{ ## crossprod among timepoints where the 'survival' is not only NA 
+                        return(crossprod(iIID[,,iObs][,colSums(!is.na(iIID[,,iObs]))>0,drop=FALSE]))
+                    }
+                })
             }
         }
 
@@ -379,7 +391,7 @@ autoplot.predictCox <- function(object,
                               digits = digits
                               )
     }
-    
+
     ## ** display
     gg.res <- predict2plot(dataL = dataL,
                            name.outcome = type,
@@ -557,7 +569,7 @@ predict2plot <- function(dataL, name.outcome,
 
             dataL[, c(paste0(name.outcome,".smooth")) := warper(.SD), by = "row"]
             if(ci){
-                warperCI <- function(data, Sigma, n.sim){ ## data <- dataL[row==1] ; Sigma <- attr(first.derivative,"vcov")[[1]]
+                warperCI <- function(data, Sigma, n.sim){ ## data <- dataL[row==2] ; Sigma <- attr(first.derivative,"vcov")[[2]]
                     ls.deriv <- lapply(1:n.sim, function(x){
                         data2 <- data.table::copy(data)
                         data2[, c(name.outcome) := mvtnorm::rmvnorm(1, mean = data[[name.outcome]], sigma = Sigma)[1,]]                        
@@ -608,10 +620,10 @@ predict2plot <- function(dataL, name.outcome,
             }else{
                 gg.base <- gg.base + ggplot2::geom_line(eval(parse(text = paste0(
                                                                        "ggplot2::aes(x = time, y = lowerBand.smooth, group = ",group.by,", color = ",group.by,", linetype = \"band\")"))),
-                                                        size = size.band)
+                                                        linewidth = size.band)
                 gg.base <- gg.base + ggplot2::geom_line(eval(parse(text = paste0(
                                                                        "ggplot2::aes(x = time, y = upperBand.smooth, group = ",group.by, ", color = ",group.by,", linetype = \"band\")"))),
-                                                        size = size.band)
+                                                        linewidth = size.band)
             }
         }else{
             if(!is.na(alpha)){
@@ -620,9 +632,9 @@ predict2plot <- function(dataL, name.outcome,
                 gg.base <- gg.base + scale_fill_manual("", values="grey12")        
             }else{
                 gg.base <- gg.base + ggplot2::geom_segment(ggplot2::aes_string(x = "time", y = "lowerBand", xend = "timeRight", yend = "lowerBand", color = "\"band\""),
-                                                           size = size.band)
+                                                           linewidth = size.band)
                 gg.base <- gg.base + ggplot2::geom_segment(ggplot2::aes_string(x = "time", y = "upperBand", xend = "timeRight", yend = "upperBand", color = "\"band\""),
-                                                           size = size.band)
+                                                           linewidth = size.band)
             }
         }
     }
@@ -635,10 +647,10 @@ predict2plot <- function(dataL, name.outcome,
             }else{
                 gg.base <- gg.base + ggplot2::geom_line(eval(parse(text = paste0(
                                                                        "ggplot2::aes(x = time, y = lowerCI.smooth, group = ",group.by,", color = ",group.by,", linetype = \"ci\")"))),
-                                                        size = size.ci)
+                                                        linewidth = size.ci)
                 gg.base <- gg.base + ggplot2::geom_line(eval(parse(text = paste0(
                                                                        "ggplot2::aes(x = time, y = upperCI.smooth, group = ",group.by,", color = ",group.by,", linetype = \"ci\")"))),
-                                                        size = size.ci)
+                                                        linewidth = size.ci)
 
             }
         }else{
@@ -649,19 +661,19 @@ predict2plot <- function(dataL, name.outcome,
 
             }else{
                 gg.base <- gg.base + ggplot2::geom_segment(ggplot2::aes_string(x = "time", y = "lowerCI", xend = "timeRight", yend = "lowerCI", color = "\"ci\""),
-                                                           size = size.ci)
+                                                           linewidth = size.ci)
                 gg.base <- gg.base + ggplot2::geom_segment(ggplot2::aes_string(x = "time", y = "upperCI", xend = "timeRight", yend = "upperCI", color = "\"ci\""),
-                                                           size = size.ci)
+                                                           linewidth = size.ci)
             }
         }
     }
     ## estimate
     if(smooth>0){
         gg.base <- gg.base + ggplot2::geom_line(mapping = ggplot2::aes_string(x = "time", y = paste0(name.outcome,".smooth"), group = group.by, color = group.by),
-                                                size = size.estimate)
+                                                linewidth = size.estimate)
     }else{
         gg.base <- gg.base + ggplot2::geom_segment(mapping = ggplot2::aes_string(x = "timeRight", y = name.outcome, xend = "time", yend = name.outcome, color = group.by),
-                                                   size = size.estimate)
+                                                   linewidth = size.estimate)
         if("status" %in% names(dataL)){
             dataL$status <- as.character(dataL$status)
             gg.base <- gg.base + ggplot2::geom_point(data = na.omit(dataL),
