@@ -3,9 +3,9 @@
 ## author: Thomas Alexander Gerds
 ## created: Jan  9 2016 (19:31) 
 ## Version: 
-## last-updated: Jan 28 2019 (14:55) 
+## last-updated: Jun 13 2024 (15:56) 
 ##           By: Thomas Alexander Gerds
-##     Update #: 287
+##     Update #: 313
 #----------------------------------------------------------------------
 ## 
 ### Commentary: 
@@ -24,13 +24,13 @@ getQuantile <- function(x,Fx,Q){
 }
 
 riskQuantile.binary <- function(DT,N,NT,NF,dolist,Q,...){
-    reference=model=ReSpOnSe=risk=cause=X=ID=NULL
+    reference=model=riskRegression_event=risk=cause=X=riskRegression_ID=NULL
     models <- unique(DT[,model])
     if (missing(Q)) Q <- c(0.05,0.25,0.5,0.75,0.95)
     else Q <- sort(Q)
-    score.event <- DT[ReSpOnSe==1,data.table(t(quantile(risk,probs=Q))),by=list(model)]
+    score.event <- DT[riskRegression_event==1,data.table(t(quantile(risk,probs=Q))),by=list(model)]
     score.event[,cause:="event"]
-    score.eventfree <- DT[ReSpOnSe==0,data.table(t(quantile(risk,probs=Q))),by=list(model)]
+    score.eventfree <- DT[riskRegression_event==0,data.table(t(quantile(risk,probs=Q))),by=list(model)]
     score.eventfree[,cause:="event-free"]
     score.overall <- DT[,data.table(t(quantile(risk,probs=Q))),by=list(model)]
     score.overall[,cause:="overall"]
@@ -41,16 +41,16 @@ riskQuantile.binary <- function(DT,N,NT,NF,dolist,Q,...){
     if (length(dolist)>0){
         contrasts <- data.table::rbindlist(lapply(dolist,function(g){
             ## from all models g[-1], substract risk of model g[1] 
-            setorder(DT,model,ID)
+            setorder(DT,model,riskRegression_ID)
             DTdiff <- DT[model%in%g[-1]]
             DTref <- rep(DT[model==g[1],risk],length(unique(DTdiff$model)))
             DTdiff[,X:=risk-DTref]
             N <- NROW(DTdiff)
             Xrange <- DTdiff[,range(X)]
             Xmed <- DTdiff[,median(X)]
-            changedist.event <- DTdiff[ReSpOnSe==1,data.table(t(quantile(X,probs=Q))),by=list(model)]
+            changedist.event <- DTdiff[riskRegression_event==1,data.table(t(quantile(X,probs=Q))),by=list(model)]
             changedist.event[,cause:="event"]
-            changedist.eventfree <- DTdiff[ReSpOnSe==0,data.table(t(quantile(X,probs=Q))),by=list(model)]
+            changedist.eventfree <- DTdiff[riskRegression_event==0,data.table(t(quantile(X,probs=Q))),by=list(model)]
             changedist.eventfree[,cause:="event-free"]
             changedist.overall <- DTdiff[,data.table(t(quantile(X,probs=Q))),by=list(model)]
             changedist.overall[,cause:="overall"]
@@ -68,7 +68,7 @@ riskQuantile.binary <- function(DT,N,NT,NF,dolist,Q,...){
 
 
 riskQuantile.survival <- function(DT,N,NT,NF,dolist,Q,...){
-    model=event=X=reference=status=times=cause=risk=Wt=WTi=cuminc=ID=NULL
+    model=event=X=reference=status=times=cause=risk=Wt=WTi=cuminc=riskRegression_ID = riskRegression_status = riskRegression_time=NULL
     models <- unique(DT[,model])
     if (missing(Q)) Q <- c(0.05,0.25,0.5,0.75,0.95) else Q <- sort(Q) ##
     ## retrospectively (looking back from time t)
@@ -105,7 +105,7 @@ riskQuantile.survival <- function(DT,N,NT,NF,dolist,Q,...){
     ##
     ## For 'event-free analyses' P(X<=x|T>t) is estimated by P(T>t|X<=x) P(X<=x)/P(T>t)
     #######
-    surv <- DT[model==models[[1]],data.table::data.table("surv"=(1/N*sum((time>times)/Wt))),by=times]
+    surv <- DT[model==models[[1]],data.table::data.table("surv"=(1/N*sum((riskRegression_time>times)/Wt))),by=times]
     surv[,cuminc:=1-surv]
     getQ.event <- function(Q,tp,X,time,status,WTi,surv){
         uX <- sort(unique(X[time<=tp & status==1]))
@@ -125,11 +125,11 @@ riskQuantile.survival <- function(DT,N,NT,NF,dolist,Q,...){
     }
     ## a <- DT[model==1]
     ## system.time(a[,getQ.eventFree(Q=Q,tp=times[1],X=risk,time=time,Wt=Wt,surv=surv)])
-    score.eventfree <- DT[,getQ.eventFree(Q=Q,tp=times,X=risk,time=time,Wt=Wt,surv=surv),by=list(model,times)]
+    score.eventfree <- DT[,getQ.eventFree(Q=Q,tp=times,X=risk,time=riskRegression_time,Wt=Wt,surv=surv),by=list(model,times)]
     ## setkey(DT,model,times)
     ## save(surv,file="~/tmp/surv.rda")
     ## save(DT,file="~/tmp/DT.rda")
-    score.event <- DT[,getQ.event(Q=Q,tp=times,X=risk,time=time,status=status,WTi=WTi,surv=surv),by=list(model,times)]
+    score.event <- DT[,getQ.event(Q=Q,tp=times,X=risk,time=riskRegression_time,status=riskRegression_status,WTi=WTi,surv=surv),by=list(model,times)]
     score.overall <- DT[,data.table(t(quantile(risk,probs=Q))),by=list(model,times)]
     score.overall[,cause:="overall"]
     colnames(score.overall) <- colnames(score.event)
@@ -140,15 +140,15 @@ riskQuantile.survival <- function(DT,N,NT,NF,dolist,Q,...){
     if (length(dolist)>0){
         contrasts <- data.table::rbindlist(lapply(dolist,function(g){
             ## from all models g[-1], substract risk of model g[1] 
-            setorder(DT,model,times,ID)
+            setorder(DT,model,times,riskRegression_ID)
             DTdiff <- DT[model%in%g[-1]]
             DTref <- rep(DT[model==g[1],risk],length(unique(DTdiff$model)))
             DTdiff[,X:=risk-DTref]
             N <- NROW(DTdiff)
             Xrange <- DTdiff[,range(X)]
             Xmed <- DTdiff[,median(X)]
-            changedist.eventfree <- DTdiff[,getQ.eventFree(Q=Q,tp=times,X=X,time=time,Wt=Wt,surv=surv),by=list(model,times)]
-            changedist.event <- DTdiff[,getQ.event(Q=Q,tp=times,X=X,time=time,status=status,WTi=WTi,surv=surv),by=list(model,times)]
+            changedist.eventfree <- DTdiff[,getQ.eventFree(Q=Q,tp=times,X=X,time=riskRegression_time,Wt=Wt,surv=surv),by=list(model,times)]
+            changedist.event <- DTdiff[,getQ.event(Q=Q,tp=times,X=X,time=riskRegression_time,status=riskRegression_status,WTi=WTi,surv=surv),by=list(model,times)]
             changedist.overall <- DTdiff[,data.table(t(quantile(X,probs=Q))),by=list(model,times)]
             changedist.overall[,cause:="overall"]
             colnames(changedist.overall) <- colnames(changedist.event)
@@ -166,7 +166,7 @@ riskQuantile.survival <- function(DT,N,NT,NF,dolist,Q,...){
 }
 
 riskQuantile.competing.risks <- function(DT,N,NT,NF,dolist,cause,states,Q,...){
-    model=event=reference=risk=X=status=times=Wt=WTi=cause=cuminc=ID=NULL
+    model=event=reference=risk=X=status=times=Wt=WTi=cause=cuminc=riskRegression_ID = riskRegression_time = riskRegression_event = NULL
     models <- unique(DT[,model])
     if (missing(Q)) Q <- c(0.05,0.25,0.5,0.75,0.95)
     else Q <- sort(Q)
@@ -177,6 +177,7 @@ riskQuantile.competing.risks <- function(DT,N,NT,NF,dolist,cause,states,Q,...){
     ##
     ## if (missing(states))
     ## states <- DT[model==models[[1]] & status!=0,sort(unique(event))]
+    states.code <- 1:length(states)
     #######
     ## Let X denote the difference between two 10-year predictions for the same subjects.
     ## This function estimates quantiles of the conditional distibutions of X given
@@ -210,13 +211,14 @@ riskQuantile.competing.risks <- function(DT,N,NT,NF,dolist,cause,states,Q,...){
     ##
     ## For 'event-free analyses' P(X<=x|T>t) is estimated by P(T>t|X<=x) P(X<=x)/P(T>t)
     #######
-    surv <- DT[model==models[[1]],data.table::data.table("surv"=(1/N*sum((time>times)/Wt))),by=times]
-    cuminc <- lapply(states,function(cc){DT[model==models[[1]],data.table::data.table("cuminc"=1/N*sum((event==cc & time<=times)/WTi)),by=times]})
+    surv <- DT[model==models[[1]],data.table::data.table("surv"=(1/N*sum((riskRegression_time>times)/Wt))),by=times]
+    cuminc <- lapply(states.code,function(cc){DT[model==models[[1]],data.table::data.table("cuminc"=1/N*sum((riskRegression_event==cc & riskRegression_time<=times)/WTi)),by=times]})
     names(cuminc) <- states
-    getQ.states <- function(Q,tp,X,time,event,WTi,cuminc,states){
+    getQ.states <- function(Q,tp,X,time,event,WTi,cuminc,states.code){
         uX <- sort(unique(X))
-        rbindlist(lapply(states,function(cause){
+        rbindlist(lapply(states.code,function(cause){
             # Note that event==cause implies status==1
+            ## the variable event has values 1,2,..,k,k+1 where k+1 is censored
             Wx <- sapply(uX,function(x){sum((X<=x & event==cause & time<=tp)/WTi)})/(N*cuminc[[cause]][times==tp,cuminc])
             qRisk <- getQuantile(x=uX,Fx=Wx,Q=Q)
             qR <- data.table(t(qRisk))
@@ -232,8 +234,8 @@ riskQuantile.competing.risks <- function(DT,N,NT,NF,dolist,cause,states,Q,...){
         qR[,cause:="event-free"]
         qR
     }
-    score.eventfree <- DT[,getQ.eventFree(Q=Q,tp=times,X=risk,time=time,Wt=Wt,surv=surv),by=list(model,times)]
-    score.states <- DT[,getQ.states(Q=Q,tp=times,X=risk,time=time,event=event,WTi=WTi,cuminc=cuminc,states=states),by=list(model,times)]
+    score.eventfree <- DT[,getQ.eventFree(Q=Q,tp=times,X=risk,time=riskRegression_time,Wt=Wt,surv=surv),by=list(model,times)]
+    score.states <- DT[,getQ.states(Q=Q,tp=times,X=risk,time=riskRegression_time,event=riskRegression_event,WTi=WTi,cuminc=cuminc,states.code=states.code),by=list(model,times)]
     score.overall <- DT[,data.table(t(quantile(risk,probs=Q))),by=list(model,times)]
     score.overall[,cause:="overall"]
     colnames(score.overall) <- colnames(score.states)
@@ -244,15 +246,15 @@ riskQuantile.competing.risks <- function(DT,N,NT,NF,dolist,cause,states,Q,...){
     if (length(dolist)>0){
         contrasts <- data.table::rbindlist(lapply(dolist,function(g){
             ## from all models g[-1], substract risk of model g[1] 
-            setorder(DT,model,times,ID)
+            setorderv(DT,cols = c("model","times","riskRegression_ID"))
             DTdiff <- DT[model%in%g[-1]]
             DTref <- rep(DT[model==g[1],risk],length(unique(DTdiff$model)))
             DTdiff[,X:=risk-DTref]
             N <- NROW(DTdiff)
             Xrange <- DTdiff[,range(X)]
             Xmed <- DTdiff[,median(X)]
-            changedist.eventfree <- DTdiff[,getQ.eventFree(Q=Q,tp=times,X=X,time=time,Wt=Wt,surv=surv),by=list(model,times)]
-            changedist.states <- DTdiff[,getQ.states(Q=Q,tp=times,X=X,time=time,event=event,WTi=WTi,cuminc=cuminc,states=states),by=list(model,times)]
+            changedist.eventfree <- DTdiff[,getQ.eventFree(Q=Q,tp=times,X=X,time=riskRegression_time,Wt=Wt,surv=surv),by=list(model,times)]
+            changedist.states <- DTdiff[,getQ.states(Q=Q,tp=times,X=X,time=riskRegression_time,event=riskRegression_event,WTi=WTi,cuminc=cuminc,states.code=states.code),by=list(model,times)]
             changedist.overall <- DTdiff[,data.table(t(quantile(X,probs=Q))),by=list(model,times)]
             changedist.overall[,cause:="overall"]
             colnames(changedist.overall) <- colnames(changedist.states)
@@ -264,6 +266,10 @@ riskQuantile.competing.risks <- function(DT,N,NT,NF,dolist,cause,states,Q,...){
             changedist
         }))
     }else contrasts <- NULL
+    if (!is.null(score))
+        score[,cause:=as.character(factor(cause,levels=c("overall",states.code,"event-free"),labels=c("overall",states,"event-free")))]
+    if (!is.null(contrasts))
+        contrasts[,cause:=as.character(factor(cause,levels=c("overall",states.code,"event-free"),labels=c("overall",states,"event-free")))]
     list(score=score,contrasts=contrasts)
 }
 
